<?xml version="1.0"?>

<st-source>
<!-- Changes in change set PVAdaptor Core-diffs-Green.15 -->
<time-stamp>From VisualWorks®, 7.3.1 of April 20, 2005 on January 13, 2010 at 10:52:42 am</time-stamp>


<class>
<name>InputSchema</name>
<environment>PROVISO.UBA</environment>
<super>PROVISO.UBA.AbstractSchema</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>uri uri2 username password recordNumber schemas onScan scannedInputs scannedInputs1 scannedInputs2 lastFoundNewInputsSeconds1 lastFoundNewInputsSeconds2 lateLimitSeconds minFileSize uriParser deleteOnAcquire localEnvironment precedence inputAgeLimit lastInputTimestamp inputTimeInterval lastProcessedTimestamp sequenceSkipDelay errorFileCount errorFileLimit retryLimit onOpen onClose onStartup acquireURI onFullDump ignoreProcessingErrors inputDescriptors handlerBlock multiFileJoin myEventTable scheduler executeCommand </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category></category>
<attributes>
<package>PVAdaptor-Schema</package>
</attributes>
</class>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>private</category>

<body package="PVAdaptor-Schema" selector="handleRecordError:">handleRecordError: ex 

	self currentObject incrementRejectedRecordCount.

	self 
		invokeSmeeCallback: #recordErrorHandler 
		withArguments: {ex description}
		ifAbsent: [].

	self adaptor ifNotNil: [:a |
		a
			logRejectedRecordNumber: self currentObject processedRecordCount
			reason: ex description
			totalThusFar: self currentObject rejectedRecordCount].</body>

<body package="PVAdaptor-Schema" selector="recordProcessingAborted:">recordProcessingAborted: anInputDescriptor 
	| stop |
	stop := SmeeDate nowUTC.
	anInputDescriptor processedTimestamp: stop.
	anInputDescriptor stopTimestamp: stop.
	self inputDescriptors at: anInputDescriptor name put: anInputDescriptor.
	^self</body>

<body package="PVAdaptor-Schema" selector="recordProcessingCompletion:">recordProcessingCompletion: anInputDescriptor 
	| stop |
	stop := SmeeDate nowUTC.
	anInputDescriptor processedTimestamp: stop.
	anInputDescriptor stopTimestamp: stop.
	anInputDescriptor logStopProcessingInput.
	self inputDescriptors at: anInputDescriptor name put: anInputDescriptor.
	^self</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="postProcessInputStreamFor:">postProcessInputStreamFor: inputDescriptor 
	
	self recordProcessingCompletion: inputDescriptor.
	inputDescriptor rejectedRecordCount &gt; 0 
		ifTrue: 
			[self incrementErrorFileCount]</body>

<body package="PVAdaptor-Schema" selector="preProcessInputStreamFor:">preProcessInputStreamFor: inputDescriptor 
	
	self lastInputTimestamp: inputDescriptor timestamp.
	inputDescriptor startTimestamp: SmeeDate nowUTC.
	self logStartProcessingInput: inputDescriptor uriInDoDirectory.
	self preRead 
		valueWithThis: self
		with: inputDescriptor</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>initialize-release</category>

<body package="PVAdaptor-Schema" selector="initialize">initialize
	
	super initialize.
	environment := Smee.SmeeEnvironment current.
	recordNumber := 0.
	scannedInputs := SmeeArray new.
	scannedInputs1 := #().
	scannedInputs2 := #().
	localEnvironment := Dictionary new.
	precedence := 0.
	inputAgeLimit := 168.		"7 days expressed in hours"
	errorFileCount := 0.
	errorFileLimit := Infinity positive.
	deleteOnAcquire := false.
	retryLimit := 5.
	acquireURI := true.
	ignoreProcessingErrors := false.
	self heartbeatPeriod: 1.		" Generate heartbeats every second "
	handlerBlock := [:eachInput | self adaptor processInputDescriptor: eachInput]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.PVLINEInputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	self currentObject: inputDescriptor.
	self environment global at: self name put: inputDescriptor.
	inputDescriptor uriInDoDirectory readStreamDo: 
		[:sourceStream :params | 
		inputDescriptor startTimestamp: SmeeDate nowUTC.
		self logStartProcessingInput: inputDescriptor uriInDoDirectory.
		self preRead 
			valueWithThis: self
			with: inputDescriptor.
		self 
			parsePvLineStream: sourceStream
			params: params
			client: client.
		self onRead 
			valueWithThis: self
			with: inputDescriptor.
		self recordProcessingCompletion: inputDescriptor].
	^self</body>

<body package="PVAdaptor-Schema" selector="parsePvLineStream:params:client:">parsePvLineStream: sourceStream params: params client: client 
	
	| stream record |
	self schemas isEmpty 
		ifTrue: 
			[stream := 
				PvLineParser new
					stream: sourceStream text;
					params: params.
			[stream atEnd] 
				whileFalse: 
					[[record := stream nextRecord.
					record isNil 
						ifFalse: 
							[client parsedObject: record.
							self onRead 
								valueWithThis: self
								with: record].
							self currentObject incrementProcessedRecordCount.] 
									on: Error
									do: [:ex | self handleRecordError: ex]]]
		ifFalse: 
			[self schemas do: 
				[:eachRecordSchema | 
				eachRecordSchema 
					parsePvLineStream: sourceStream
					params: params
					client: client]]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.SQLInputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	| stream |
	self currentObject: inputDescriptor.
	self environment global at: self name put: inputDescriptor.
	inputDescriptor uriInDoDirectory readStreamDo: 
		[:sourceStream :params | 
		inputDescriptor startTimestamp: SmeeDate nowUTC.
		self logStartProcessingInput: inputDescriptor uriInDoDirectory.
		self preRead valueWithThis: self with: inputDescriptor.
		stream := 
			self inputStreamClass 
				on: sourceStream
				schema: self
				params: params.
		(self recordSchemaFromLocalName: inputDescriptor localNameString)
			parseHeaderRecordFrom: stream;
			parseAllFrom: stream
				notifying: client].
	self onRead valueWithThis: self with: inputDescriptor.
	self recordProcessingCompletion: inputDescriptor.
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.AlcatelMRBalancerInputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	| balancer tmpStrm urix nullDelimitedString |
	(self shouldProcessAlcatelMRInput: inputDescriptor) 
		ifFalse: [^self].
	self recordNumber: 0.
	self enqueueAlcatelAvailFilesToBeMovedForInput: inputDescriptor.
	(urix := inputDescriptor uriInDoDirectory) exists 
		ifTrue: 
			[self lastInputTimestamp: inputDescriptor timestamp.
			self updateLastProcessedTimestamp.
			self currentObject: inputDescriptor.
			self environment global at: self name put: inputDescriptor.
			urix readStreamDo: 
				[:sourceStream :params | 
				inputDescriptor startTimestamp: SmeeDate nowUTC.
				self logStartProcessingInput: inputDescriptor uriInDoDirectory.
				self preRead valueWithThis: self with: inputDescriptor.
				params at: #InputDescriptor put: inputDescriptor.
				balancer := 
					AlcatelMRBalancer new
						owner: self;
						stream: sourceStream text;
						params: params.
				[balancer nameMap: self getNameMap.
				balancer atEnd ifFalse: [balancer nextHeader].
				nullDelimitedString := PvNullDelimitedString new.
				tmpStrm := (String new: 2000) writeStream.
				[balancer atEnd] 
					whileFalse: 
						[[tmpStrm reset.
						balancer nextRecordAsNullDelimitedStringOnto: tmpStrm.
						nullDelimitedString string: tmpStrm collection from: 1 to: tmpStrm position.
						balancer nextPutRecord: nullDelimitedString.
						self incrementRecordNumber.
						client parsedObject: nullDelimitedString] 
									on: Error
									do: [:ex | self handleRecordError: ex]]] 
										ensure: [balancer closeAll].
				self onRead valueWithThis: self with: inputDescriptor.
				self recordProcessingCompletion: inputDescriptor.
				self recordNumber: 0]]
		ifFalse: 
			["We should not ever get here"
			self system 
				logWarning: #ALCATEL_MR_NOT_FOUND
				messageId: #DL39215
				text: 
					'Alcatel MR file ' , inputDescriptor name , 
						' does not exist -- deleting file descriptor'.
			self deleteInputDescriptor: inputDescriptor].
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.CWMInputSchema</class-id> <category>parsing</category>

<body package="PV-CWM-Application" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client
	
	self currentObject: inputDescriptor.
	SmeeEnvironment current global at: self name put: inputDescriptor.
      self resetProcessedRecordCounts.
	self recordNumber: 0.
	inputDescriptor rejectedRecordCount: 0.
	inputDescriptor uriInDoDirectory exists 
		ifTrue: 
			[inputDescriptor uriInDoDirectory readStreamDo: 
				[:sourceStream :params | 
				self lastInputTimestamp: inputDescriptor timestamp.
				inputDescriptor startTimestamp: SmeeDate nowUTC.
				self logStartProcessingInput: inputDescriptor uriInDoDirectory.
				self preRead valueWithThis: self with: inputDescriptor.

				sourceStream binary.
				self processor parser stream: sourceStream.
				self processor inputDescriptor: inputDescriptor.
				[self processor reduce]
					on: Error
					do: [ :ex | self logParsingError: inputDescriptor].

				self processor parser stream: nil.
				self processor currentRecord: nil.

				self recordProcessingCompletion: inputDescriptor.
				self deleteInputDescriptor: inputDescriptor.
				inputDescriptor rejectedRecordCount &gt; 0 ifTrue: 
					[inputDescriptor rejectedRecordCount: 0.
					self incrementErrorFileCount]]]
		ifFalse: 
			[self system 
				logWarning: #INPUT_MISSING
				messageId: #DL39214 findNextUnusedMessageId
				text: 'Input ' , inputDescriptor name , ' not found'.
			self recordProcessingAborted: inputDescriptor].
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.SAMPassThruSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	| uriInDo cNumber |
	(self shouldProcessInput: inputDescriptor) ifFalse: [^self].
	self currentObject: inputDescriptor.
	self lastInputTimestamp: inputDescriptor timestamp.
	self environment global at: self name put: inputDescriptor.
	inputDescriptor startTimestamp: SmeeDate nowUTC.
	self logStartProcessingInput: inputDescriptor uriInDoDirectory.

	uriInDo := inputDescriptor uriInDoDirectory.
	(cNumber := self collectorNumberFor: inputDescriptor) = -1
		ifTrue: [^self error: 'no collector number for sam stats file'].
	inputDescriptor
		addCommitMessage: 
			(MessageSend 
				receiver: nil
				selector: #linkFrom:to:
				arguments: 
					(Array 
						with: uriInDo
						with: 
							(self fileInterface outputDirectory, 
								((uriInDo tail copyReplaceAll: ('.', self fileExtension) with: ''), '.', cNumber asString, '.', self fileExtension)))).
	self onRead valueWithThis: self with: inputDescriptor.
	self recordProcessingCompletion: inputDescriptor.
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>parsing callbacks</category>

<body package="NGUBA-Application" selector="finishParsingFor:">finishParsingFor: anInputDescriptor 
	
	anInputDescriptor stopTimestamp: SmeeDate nowUTC.
	anInputDescriptor stream: nil.
	self sendEndHeartbeat: anInputDescriptor timestamp.
	self moveFileToDoneFor: anInputDescriptor.
	self saveStateFor: anInputDescriptor.
	anInputDescriptor logStopProcessingInput.
	" Send heartbeat downstream "</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>state recovery</category>

<body package="NGUBA-Application" selector="saveStateFor:">saveStateFor: anInputDescriptor 
	" Transfer all stats from the schema to the input descriptor and commit it "
	
	anInputDescriptor maxTimestamp: self maxTimestamp.
	anInputDescriptor save</body>
</methods>


<methods>
<class-id>PROVISO.UBA.CWMPassThruSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	| uriInDo cNumber |
	(self shouldProcessInput: inputDescriptor) ifFalse: [^self].
	self currentObject: inputDescriptor.
	self lastInputTimestamp: inputDescriptor timestamp.
	self environment global at: self name put: inputDescriptor.
	inputDescriptor startTimestamp: SmeeDate nowUTC.
	self logStartProcessingInput: inputDescriptor uriInDoDirectory.

	uriInDo := inputDescriptor uriInDoDirectory.
	(cNumber := self collectorNumberFor: inputDescriptor) = -1
		ifTrue: [^self error: 'no collector number for cwm stats file'].
	inputDescriptor
		addCommitMessage: 
			(MessageSend 
				receiver: nil
				selector: #linkFrom:to:
				arguments: 
					(Array 
						with: uriInDo
						with: 
							(self fileInterface outputDirectory, 
								(uriInDo tail, '.', cNumber asString, '.cwm')))).
	self onRead valueWithThis: self with: inputDescriptor.
	self recordProcessingCompletion: inputDescriptor.
	self enqueueAvailFilesToBeMovedForInput: inputDescriptor.
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.UBAServiceInstance</class-id> <category>testing</category>

<body package="PVAdaptor-Application" selector="processedRecordCount">processedRecordCount
	"USE ONLY FOR TESTING!!!!"
	^self inputDescriptors inject: 0 into: [:sum :each | sum + each processedRecordCount]</body>
</methods>

<methods>
<class-id>PROVISO.UBA.UBAServiceInstance</class-id> <category>adapting</category>

<body package="PVAdaptor-Application" selector="commit:">commit: inputSourceDescriptor 
	
	(inputSourceDescriptor notNil and: 
		[inputSourceDescriptor ignore not and: [self shouldCommit: inputSourceDescriptor]]) 
			ifTrue: 
				[self system extendedStatus: 'Commiting ' , inputSourceDescriptor localName.
				"send #flushAll instead of commitAll to avoid doing duplicate 
				moveTempFilesToOutputUsing"
				self modelInterface 
					ifNotNil: [:mi | mi flushAll].
				self transactionDo: 
					[:log | 
					inputSourceDescriptor addCommitMessagesTo: log.
					self moveTempFilesToOutputUsing: log].
				^self onCommit valueWithThis: self]</body>

<body package="PVAdaptor-Application" selector="shouldCommit:">shouldCommit: anInputSourceDescriptor
	(self showOnly or: [self shouldDumpXML]) 
		ifTrue: [^false].
	^(anInputSourceDescriptor rejectedRecordCount = 0 or: [self failOnRejectedRecords not])</body>
</methods>


<methods>
<class-id>PROVISO.UBA.AlcatelMRBalancer</class-id> <category>private</category>

<body package="PVAdaptor Core" selector="nextPutRecord:objectNameIndex:fieldsAndIndexes:">nextPutRecord: aNullDelimitedString
	objectNameIndex: nameIndex
	fieldsAndIndexes: fieldNamesAndIndexes
	
	| objectName nameMapEntry id collectorNumber outputStream |
	(aNullDelimitedString at: nameIndex equals: '-') ifTrue: [^self].
	objectName := aNullDelimitedString at: nameIndex.
	nameMapEntry := 
		nameMap 
			at: objectName
			ifAbsent: 
				[PvException generic raiseErrorString: 
					'No name map entry found for ' , objectName smeeToString].
	id := nameMapEntry key.
	collectorNumber := nameMapEntry value.
	outputStream := 
		outputStreams 
			at: collectorNumber
			ifAbsent: [nil].
	outputStream isNil 
		ifTrue: 
			[PvException generic raiseErrorString: 
				'No output stream found for collector #' , collectorNumber smeeToString].
	fieldNamesAndIndexes do: 
		[:each | 
		aNullDelimitedString write: each value on: outputStream.
		outputStream nextPut: $,].
	owner currentObject incrementProcessedRecordCount.
	outputStream
		print: id;
		cr</body>
</methods>


<methods>
<class-id>PROVISO.UBA.BOFInputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="handle:using:">handle: record using: client 
	
	| smeeRec |
	client parsedObject: record.
	self incrementRecordNumber.
	smeeRec := 
		self 
			convertToSmee: record
			using: client pvModel.
	smeeRec schema: self.
	^self schemas do: 
		[:eachRecordSchema | 
			eachRecordSchema onRead 
					valueWithThis: eachRecordSchema
					with: smeeRec.
			self currentObject incrementProcessedRecordCount]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.RecordSchema</class-id> <category>events</category>

<body package="PVAdaptor-Schema" selector="triggerOnReadWith:">triggerOnReadWith: aFieldOrRecord 
	
	[self rootSchema currentObject incrementProcessedRecordCount] 
		on: MessageNotUnderstood
		do: [:ex | "do nothing"].
	^super triggerOnReadWith: aFieldOrRecord</body>
</methods>


<methods>
<class-id>PROVISO.UBA.PVLINERecordSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parsePvLineStream:params:client:">parsePvLineStream: sourceStream params: params client: client 
	
	| stream record |
	stream := 
		PvLineParser new
			stream: sourceStream text;
			params: params.
	[stream atEnd] 
		whileFalse: 
			[[record := stream nextRecord.
			record isNil 
				ifFalse: 
					[client parsedObject: record.
					self onRead 
						valueWithThis: self
						with: record].
					self rootSchema currentObject incrementProcessedRecordCount] 
							on: Error
							do: [:ex | self handleRecordError: ex]]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.CDWPassThruSchema</class-id> <category>accessing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	| uriInDo cNumber |
	(self shouldProcessInput: inputDescriptor) 
		ifTrue: 
			[self currentObject: inputDescriptor.
			self lastInputTimestamp: inputDescriptor timestamp.
			self environment global at: self name put: inputDescriptor.
			inputDescriptor startTimestamp: SmeeDate nowUTC.
			self logStartProcessingInput: inputDescriptor uriInDoDirectory.
			uriInDo := inputDescriptor uriInDoDirectory.
			(cNumber := self collectorNumberFor: inputDescriptor) = -1 
				ifTrue: [^self error: 'no collector number for cwm stats file'].
			inputDescriptor addCommitMessage: (MessageSend 
						receiver: nil
						selector: #linkFrom:to:
						arguments: (Array with: uriInDo
								with: self fileInterface outputDirectory 
										, (uriInDo tail , '.' , cNumber asString , '.cwm'))).
			self onRead valueWithThis: self with: inputDescriptor.
			self recordProcessingCompletion: inputDescriptor]
		ifFalse: 
			[inputDescriptor doNotProcess].
	self enqueueAvailFilesToBeMovedForInput: inputDescriptor.
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.NoneInputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	self currentObject: inputDescriptor.
	self environment global at: self name put: inputDescriptor.
	inputDescriptor startTimestamp: SmeeDate nowUTC.
	self logStartProcessingInput: inputDescriptor uriInDoDirectory.
	self onRead valueWithThis: self with: inputDescriptor.
	self recordProcessingCompletion: inputDescriptor.
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.InputDescriptor</class-id> <category>accessing</category>

<body package="PVAdaptor-Schema" selector="incrementProcessedRecordCount">incrementProcessedRecordCount

	processedRecordCount := processedRecordCount + 1</body>

<body package="PVAdaptor-Schema" selector="incrementRejectedRecordCount">incrementRejectedRecordCount
	rejectedRecordCount := rejectedRecordCount + 1</body>

<body package="PVAdaptor-Schema" selector="recordsPerMinute">recordsPerMinute
	| seconds |
	seconds := self elapsedSeconds.
	^seconds isZero
		ifTrue: [4.2d42]
		ifFalse: [((self processedRecordCount / seconds) * 60) asFloat]</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputDescriptor</class-id> <category>initialize-release</category>

<body package="PVAdaptor-Schema" selector="initialize">initialize
	
	super initialize.
	ignore := false.
	self resetCommitMessages.
	inputSourceStatus := String new.
	noNewInputCount := 0.
	processedRecordCount := 0.
	rejectedRecordCount := 0.</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputDescriptor</class-id> <category>logging</category>

<body package="PVAdaptor-Schema" selector="logPerformanceStats">logPerformanceStats

	self schema adaptor ifNotNil: [
		self system 
			logInfo: #PERF_INPUT_PROCESSING
			template: 'Processed &lt;1p&gt; records in &lt;2p&gt; seconds (&lt;3p&gt; records/minute); rejected &lt;4p&gt; records'
			arguments: 
				{self processedRecordCount.
				self elapsedSeconds.
				self recordsPerMinute.
				self rejectedRecordCount}]</body>

<body package="PVAdaptor-Schema" selector="logStopProcessingInput">logStopProcessingInput
	&lt;comment: 'The adaptor completed processing of the named input' 
	probableCause: 'Normal if 0 rejected records' 
	suggestedAction: 'If records were rejected, this could indicate a malformed file, file with missing fields (specified in an object adaptor and with no "default value" or problem with adaptor configuration'&gt; 
	
	self system 
		logInfo: #STOP_INPUT
		messageId: #DL39806
		template: ##('Completed processing input: &lt;1s:input_name&gt; uri: &lt;2s:uri&gt;' asLogTemplate)
		arguments: [{self schema name. self uriInDoDirectory asStringWithObfuscatedPassword}].
	self logPerformanceStats.</body>
</methods>


<methods>
<class-id>PROVISO.UBA.UBANPServiceInstance</class-id> <category>adapting</category>

<body package="PVAdaptor-Application" selector="commit:">commit: inputSourceDescriptor 
	
	inputSourceDescriptor isNil 
		ifTrue: 
			[self transactionDo: 
				[:log | self moveTempFilesToOutputUsing: log].
			self onCommit valueWithThis: self]
		ifFalse: 
			[(inputSourceDescriptor ignore not and: [self shouldCommit: inputSourceDescriptor]) 
				ifTrue: 
					[self system extendedStatus: 'Commiting ' , inputSourceDescriptor localName.
					self objectBuilders do: [:each | each prepare].
					self objectBuilders do: [:each | each finish].
					self  transactionDo: 
						[:log | 
						inputSourceDescriptor addCommitMessagesTo: log.
						self moveTempFilesToOutputUsing: log].
					self onCommit valueWithThis: self]]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.PassThruInputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	self currentObject: inputDescriptor.
	self environment global at: self name put: inputDescriptor.
	inputDescriptor startTimestamp: SmeeDate nowUTC.
	self logStartProcessingInput: inputDescriptor uriInDoDirectory.
	self lastInputTimestamp: inputDescriptor timestamp.
	inputDescriptor enqueueLinkToOutput.
	self onRead valueWithThis: self with: inputDescriptor.
	self recordProcessingCompletion: inputDescriptor.
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.IdMapSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	| journal |
	inputDescriptor uriInDoDirectory exists 
		ifTrue: 
			[self currentObject: inputDescriptor.
			self environment global at: self name put: inputDescriptor.
			inputDescriptor startTimestamp: SmeeDate nowUTC.
			self logStartProcessingInput: inputDescriptor uriInDoDirectory.
			self preRead valueWithThis: self with: inputDescriptor.
			"Note: there is a chance here that we'll do some extra work...if multiple idmap files are in the do directory (which might happen if BLB/UBA has not been running for a while), we'll be reading the changes in all of them...the only harm in that is the extra work that we will be doing.  A solution would be to provide a mechanism in PvrJournal to only open a journal on a single file (as opposed to finding all log file matchine the name template)."
			journal := PvrJournal 
				recoverFromDirectory: inputDescriptor filenameInDoDirectory directory
				named: 'idmap'
				logNameTemplate: '&lt;1s&gt;~&lt;2s&gt;~&lt;3s&gt;.&lt;4s&gt;' 
				nameSeparator: '~'
				timeBlock: PvrExternalStreamFactory defaultTimeBlock.
			journal materializeCallback: [ :strm | strm upTo: Character lf ].
			[self readIdMapEntriesFrom: journal readStream]
				ensure: [journal close].
			self onRead valueWithThis: self with: inputDescriptor.
			self recordProcessingCompletion: inputDescriptor]
		ifFalse: [self ignoreInputDescriptor: inputDescriptor].
	self deleteInputDescriptor: inputDescriptor.
	^self</body>
</methods>


<methods>
<class-id>PROVISO.UBA.IdMapBLBSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="parseInput:notifying:">parseInput: inputDescriptor notifying: client 
	
	| journal uriInDo |
	uriInDo := inputDescriptor uriInDoDirectory.
	uriInDo exists 
		ifTrue: 
			[self currentObject: inputDescriptor.
			self environment global at: self name put: inputDescriptor.
			inputDescriptor startTimestamp: SmeeDate nowUTC.
			self logStartProcessingInput: inputDescriptor uriInDoDirectory.
			self preRead valueWithThis: self with: inputDescriptor.
			"Note: there is a chance here that we'll do some extra work...if multiple idmap files are in the do directory (which might happen if BLB/UBA has not been running for a while), we'll be reading the changes in all of them...the only harm in that is the extra work that we will be doing.  A solution would be to provide a mechanism in PvrJournal to only open a journal on a single file (as opposed to finding all log file matchine the name template)."
			journal := PvrJournal 
				recoverFromDirectory: inputDescriptor filenameInDoDirectory directory
				named: 'idmap'
				logNameTemplate: '&lt;1s&gt;~&lt;2s&gt;~&lt;3s&gt;.&lt;4s&gt;' 
				nameSeparator: '~'
				timeBlock: PvrExternalStreamFactory defaultTimeBlock.
			journal materializeCallback: [ :strm | strm upTo: Character lf ].
			[self readIdMapEntriesFrom: journal readStream]
				ensure: [journal close].
			self knownCollectorNumbers do: 
				[ :cNumber |
				inputDescriptor addCommitMessage: 
					(MessageSend 
						receiver: nil
						selector: #linkFrom:to:
						arguments: 
							(Array 
								with: uriInDo
								with: 
									(self fileInterface outputDirectory, (cNumber asString, '.', uriInDo tail))))].
			self onRead valueWithThis: self with: inputDescriptor.
			self recordProcessingCompletion: inputDescriptor]
		ifFalse: [self ignoreInputDescriptor: inputDescriptor].
	self deleteInputDescriptor: inputDescriptor.
	^self</body>
</methods>



</st-source>
