<?xml version="1.0"?>

<st-source>
<!-- Changes in change set PATCH-RA802142 -->
<time-stamp>From VisualWorks®, 7.3.1 of April 20, 2005 on January 22, 2010 at 11:23:56 am</time-stamp>


<methods>
<class-id>PROVISO.UBA.UBAOperator</class-id> <category>state recovery</category>

<body package="NGUBA-Application" selector="earliestOr:from:">earliestOr: recoverAtOrEarlier from: eachOperator 
	
	^eachOperator trueClients 
		inject: recoverAtOrEarlier
		into: 
			[:min :eachOp1 | 
			eachOp1 startOfPeriod 
				ifNil: [min]
				ifNotNil: [:value | min min: value]]</body>

<body package="NGUBA-Application" selector="recoverPersistentStateGlobally">recoverPersistentStateGlobally
	" Recover state of all operators in the graph. Start with egress operators and recover their state, 
	then proceed to their predecessors in breadth-first order. For each operator recover it at earliest period of its successors "
	
	| sortedForRecovery recoverAtOrEarlier |
	sortedForRecovery := self operatorsFromEgressToIngressInBreadthFirstOrder.
	recoverAtOrEarlier := Infinity positive.
	sortedForRecovery do: 
		[:eachOperator | 
		recoverAtOrEarlier := 
			self 
				earliestOr: recoverAtOrEarlier
				from: eachOperator.
		eachOperator parent ifNotNil: 
			[:parentOp | 
			recoverAtOrEarlier := 
				self 
					earliestOr: recoverAtOrEarlier
					from: parentOp].
		eachOperator recoverAtOrBefore: recoverAtOrEarlier]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.InputDescriptorScheduler</class-id> <category>public</category>

<body package="PVAdaptor-Schema" selector="processInputs">processInputs
	
	| acquiredAnything |
	[acquiredAnything := self acquireOnNoNewInputDo: [:each | each incrementNoNewInputCount].
	self schedule.
	acquiredAnything] whileTrue</body>
</methods>

<remove-selector>
<class-id>Root.Smalltalk.PROVISO.UBA.InputDescriptorScheduler</class-id> <selector>processInputsForAdaptor:</selector>
</remove-selector>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="inputStreamFor:do:">inputStreamFor: inputDescriptor do: aBlock 
	
	self prepareToProcessInputDescriptor: inputDescriptor.
	inputDescriptor uriInDoDirectory exists 
		ifTrue: 
			[(self streamFactoryFor: inputDescriptor) readStreamDo: 
				[:sourceStream :params | 
				inputDescriptor stream: sourceStream.
				aBlock 
					value: sourceStream
					value: params]]
		ifFalse: [self logInputDescriptorMissingFor: inputDescriptor]</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>testing</category>

<body package="PVAdaptor-Schema" selector="isValidInputDescriptor:">isValidInputDescriptor: anInputDescriptor 
	" Should be suitable for all file-based input schemas "
	
	^self isValidFileInputDescriptor: anInputDescriptor</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>logging</category>

<body package="PVAdaptor-Schema" selector="logInputDescriptorMissingFor:">logInputDescriptorMissingFor: inputDescriptor 
	
	| locatorString |
	locatorString := 
		[inputDescriptor filenameInDoDirectory asString] 
			on: Error
			do: [:ex | inputDescriptor name].
	self system 
		logWarning: #INPUT_MISSING
		messageId: #DL39214
		text: 'Input file ' , locatorString , ' not found'.
	self recordProcessingAborted: inputDescriptor</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>initialize-release</category>

<body package="PVAdaptor-Schema" selector="positionToStartProcessingAfterTimestamp:">positionToStartProcessingAfterTimestamp: aNumber 
	
	self myInputDescriptorsDo: 
		[:descriptor | 
		(descriptor alreadyProcessed and: [descriptor schema == self]) 
			ifTrue: 
				[descriptor timestamp asUTCSeconds &gt;= aNumber 
					ifTrue: 
						[self system 
							logWarning: #FILE_TO_BE_REPROCESSED
							messageId: #DC10134
							template: 'File &lt;1s&gt; is to be reprocessed because processing starts at &lt;2p&gt;'
							arguments: {descriptor localName. (PvTimestamp fromSeconds: aNumber)}.
						descriptor processedTimestamp: nil.
						self adaptor updateInputDescriptor: descriptor]]]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.SQLInputSchema</class-id> <category>testing</category>

<body package="PVAdaptor-Schema" selector="isValidInputDescriptor:">isValidInputDescriptor: anInputDescriptor 
	" Can't see how this can be wrong "
	
	^true</body>
</methods>


<methods>
<class-id>PROVISO.UBA.BOFInputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="inputStreamFor:do:">inputStreamFor: inputDescriptor do: aBlock 
	
	| stream sourceURI destURI |
	self prepareToProcessInputDescriptor: inputDescriptor.
	inputDescriptor uriInDoDirectory exists 
		ifFalse: 
			[inputDescriptor uriInDoneDirectory exists 
				ifTrue: 
					[inputDescriptor fileInterface transactionDo: 
						[:log | 
						sourceURI := inputDescriptor uriInDoneDirectory.
						destURI := inputDescriptor uriInDoDirectory.
						log fileInterface 
							moveFilename: sourceURI
							toFilename: destURI
							using: log]]].
	inputDescriptor uriInDoDirectory exists 
		ifTrue: 
			[stream := (PvDataFile fromFilename: inputDescriptor uriInDoDirectory asFilename) readStream.
			[aBlock 
				value: stream
				value: Dictionary new] 
					ensure: [stream close]]
		ifFalse: [self logInputDescriptorMissingFor: inputDescriptor]</body>
</methods>


<methods>
<class-id>PROVISO.UBA.InputDescriptor</class-id> <category>accessing</category>

<body package="PVAdaptor-Schema" selector="filenameInDoneDirectory">filenameInDoneDirectory
	
	^(self fileInterface doneDirectoryForSchema: self schema) , self localName</body>

<body package="PVAdaptor-Schema" selector="stopTimestamp:">stopTimestamp: anObject
	&lt;setterNamed: #stopTimestamp&gt;
	stopTimestamp := anObject.
	self processedTimestamp: anObject</body>

<body package="PVAdaptor-Schema" selector="uriInDoneDirectory">uriInDoneDirectory
	
	^self filenameInDoneDirectory asFilename asURI</body>
</methods>


<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>parsing callbacks</category>

<body package="NGUBA-Application" selector="moveFileToDoFor:">moveFileToDoFor: anInputDescriptor
	"move processed file from done to do dir"
	
	self transactionDo: [:log | self moveFileToDoFor: anInputDescriptor using: log]</body>

<body package="NGUBA-Application" selector="moveFileToDoFor:using:">moveFileToDoFor: anInputDescriptor using: log 
	
	| sourceURI destURI |
	sourceURI := anInputDescriptor uriInDoneDirectory.
	destURI := anInputDescriptor uriInDoDirectory.
	log fileInterface 
		moveFilename: sourceURI
		toFilename: destURI
		using: log</body>

<body package="NGUBA-Application" selector="moveFileToDoneFor:using:">moveFileToDoneFor: anInputDescriptor using: log 
	
	| sourceURI destURI |
	sourceURI := anInputDescriptor uriInDoDirectory.
	destURI := anInputDescriptor uriInDoneDirectory.
	log fileInterface 
		moveFilename: sourceURI
		toFilename: destURI
		using: log</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>private</category>

<body package="NGUBA-Application" selector="process:ifValidDo:">process: anInputDescriptor ifValidDo: aBlock 
	
	| currentDesc |
	currentDesc := 
		self schema inputDescriptors 
			at: anInputDescriptor name
			ifAbsent: [anInputDescriptor].
	currentDesc alreadyProcessed 
		ifTrue: [^self].
	currentDesc ignore 
		ifTrue: 
			[currentDesc doNotProcess.
			^self].
	(self schema isValidInputDescriptor: anInputDescriptor) 
		ifFalse: 
			[anInputDescriptor uriInDoneDirectory exists 
				ifTrue: [self moveFileToDoFor: anInputDescriptor].
			(self schema isValidInputDescriptor: anInputDescriptor) 
				ifFalse: 
					[self schema logInputDescriptorMissingFor: anInputDescriptor.
					anInputDescriptor doNotProcess.
					^self]].
	(self hasValidTimestampForDescriptor: anInputDescriptor) 
		ifFalse: [self logNoTimestampForDescriptor: anInputDescriptor].
	self descriptor: anInputDescriptor.
	anInputDescriptor adaptor: self.
	[aBlock value: currentDesc] 
		ensure: 
			[self descriptor: nil.
			anInputDescriptor adaptor: nil]</body>
</methods>



</st-source>
